@@ -49,19 +49,12 @@ class Context(object):
     This context can be locked, so that changing the members is prevented.
     """
 
-    DEFAULT_LOG_SPACE = 'logs'
-    DEFAULT_SOURCE_SPACE = 'src'
-    DEFAULT_BUILD_SPACE = 'build'
-    DEFAULT_DEVEL_SPACE = 'devel'
-    DEFAULT_INSTALL_SPACE = 'install'
+    CATKIN_SPACES_GROUP = 'catkin_tools.spaces'
+
+    SPACES = {}
 
     STORED_KEYS = [
         'extend_path',
-        'source_space',
-        'log_space',
-        'build_space',
-        'devel_space',
-        'install_space',
         'devel_layout',
         'install',
         'isolate_install',
@@ -73,14 +66,65 @@ class Context(object):
         'catkin_make_args',
         'whitelist',
         'blacklist',
+        'authors',
+        'maintainers',
+        'licenses',
+        'extends',
     ]
 
-    KEYS = STORED_KEYS + [
+    EXTRA_KEYS = [
         'workspace',
         'profile',
         'space_suffix',
     ]
 
+    KEYS = []
+
+    @classmethod
+    def _create_space_methods(cls, space):
+        def space_abs_getter(self):
+            return getattr(self, '__%s_space_abs' % space)
+
+        def space_getter(self):
+            return getattr(self, '__%s_space' % space)
+
+        def space_setter(self, value):
+            if self.__locked:
+                raise RuntimeError("Setting of context members is not allowed while locked.")
+            setattr(self, '__%s_space' % space, value)
+            setattr(self, '__%s_space_abs' % space, os.path.join(self.__workspace, value))
+
+        def space_exists(self):
+            """
+            Returns true if the space exists.
+            """
+            space_abs = getattr(self, '__%s_space_abs' % space)
+            return os.path.exists(space_abs) and os.path.isdir(space_abs)
+
+        setattr(cls, '%s_space' % space, property(space_getter, space_setter))
+        setattr(cls, '%s_space_abs' % space, property(space_abs_getter))
+        setattr(cls, '%s_space_exists' % space, space_exists)
+
+    @classmethod
+    def setup_space_keys(cls):
+        """
+        To be called one time on initial use. Initializes the SPACE_KEYS
+        class members and associated member functions based on available
+        space plugins.
+        """
+        if cls.KEYS:
+            return
+
+        from pkg_resources import iter_entry_points
+
+        for entry_point in iter_entry_points(group=cls.CATKIN_SPACES_GROUP):
+            ep_dict = entry_point.load()
+            cls.STORED_KEYS.append(entry_point.name + '_space')
+            cls.SPACES[entry_point.name] = ep_dict
+            cls._create_space_methods(entry_point.name)
+
+        cls.KEYS = cls.STORED_KEYS + cls.EXTRA_KEYS
+
     @classmethod
     def load(
         cls,
@@ -125,6 +169,7 @@ class Context(object):
         :returns: A potentially valid Context object constructed from the given arguments
         :rtype: Context
         """
+        Context.setup_space_keys()
 
         # Initialize dictionary version of opts namespace
         opts_vars = vars(opts) if opts else {}
@@ -149,7 +194,28 @@ class Context(object):
 
         # Get the metadata stored in the workspace if it was found
         if workspace:
-            config_metadata = metadata.get_metadata(workspace, profile, 'config')
+            key_origins = {}
+            visited_profiles = []
+
+            def get_metadata_recursive(profile):
+                config_metadata = metadata.get_metadata(workspace, profile, 'config')
+                visited_profiles.append(profile)
+                if "extends" in config_metadata.keys() and config_metadata["extends"] is not None:
+                    base_profile = config_metadata["extends"]
+                    if base_profile in visited_profiles:
+                        raise IOError("Profile dependency circle detected at dependency from '%s' to '%s'!"
+                                      % (profile, base_profile))
+                    base = get_metadata_recursive(base_profile)
+                    base.update(config_metadata)
+                    for key in config_metadata.keys():
+                        key_origins[key] = profile
+                    return base
+                for key in config_metadata.keys():
+                    key_origins[key] = profile
+                return config_metadata
+
+            config_metadata = get_metadata_recursive(profile)
+
             context_args.update(config_metadata)
 
         # User-supplied args are used to update stored args
@@ -164,8 +230,14 @@ class Context(object):
                         context_args[k] = [w for w in context_args[k] if w not in v]
                     else:
                         context_args[k] = v
+                    if workspace:
+                        key_origins[k] = profile
                 else:
                     context_args[k] = v
+                    if workspace:
+                        key_origins[k] = profile
+
+        context_args["key_origins"] = key_origins
 
         # Create the build context
         ctx = Context(**context_args)
@@ -179,11 +251,28 @@ class Context(object):
     @classmethod
     def save(cls, context):
         """Save a context in the associated workspace and profile."""
-        metadata.update_metadata(
-            context.workspace,
-            context.profile,
-            'config',
-            context.get_stored_dict())
+
+        data = context.get_stored_dict()
+        files = {}
+
+        def save_in_file(file, key, value):
+            if file in files.keys():
+                files[file][key] = value
+            else:
+                files[file] = {key: value}
+
+        for key, val in data.items():
+            if key in context.key_origins:
+                save_in_file(context.key_origins[key], key, val)
+            else:
+                save_in_file(context.profile, key, val)
+
+        for profile, content in files.items():
+            metadata.update_metadata(
+                context.workspace,
+                profile,
+                'config',
+                content)
 
     def get_stored_dict(self):
         """Get the context parameters which should be stored persistently."""
@@ -194,11 +283,6 @@ class Context(object):
         workspace=None,
         profile=None,
         extend_path=None,
-        source_space=None,
-        log_space=None,
-        build_space=None,
-        devel_space=None,
-        install_space=None,
         devel_layout=None,
         install=False,
         isolate_install=False,
@@ -211,6 +295,11 @@ class Context(object):
         space_suffix=None,
         whitelist=None,
         blacklist=None,
+        authors=None,
+        maintainers=None,
+        licenses=None,
+        extends=None,
+        key_origins={},
         **kwargs
     ):
         """Creates a new Context object, optionally initializing with parameters
@@ -256,33 +345,49 @@ class Context(object):
         :param blacklist: a list of packages to ignore by default
         :type blacklist: list
         :raises: ValueError if workspace or source space does not exist
+        :type authors: list
+        :param authors: a list of default authors
+        :type maintainers: list
+        :param maintainers: a list of default maintainers
+        :type licenses: list
+        :param licenses: a list of default licenses
+        :type extends: string
+        :param extends: the name of a profile to use as a base and inherit settings from
         """
         self.__locked = False
 
+        # Validation is done on assignment
+        self.workspace = workspace
+
+        self.extend_path = extend_path if extend_path else None
+        self.key_origins = key_origins
+
+        self.profile = profile
+
+        # Handle *space assignment and defaults
+        for space, space_dict in Context.SPACES.items():
+            key_name = space + '_space'
+            default = space_dict['default']
+            value = kwargs.pop(key_name, default)
+            if value == default and space_suffix and space != 'source':
+                value += space_suffix
+            setattr(self, key_name, value)
+
         # Check for unhandled context options
         if len(kwargs) > 0:
             print('Warning: Unhandled config context options: {}'.format(kwargs), file=sys.stderr)
 
-        # Validation is done on assignment
-        # Handle *space assignment and defaults
-        self.workspace = workspace
-
-        self.extend_path = extend_path if extend_path else None
-        ss = '' if space_suffix is None else space_suffix
-
-        self.profile = profile
-
-        self.source_space = Context.DEFAULT_SOURCE_SPACE if source_space is None else source_space
-        self.log_space = Context.DEFAULT_LOG_SPACE + ss if ss or log_space is None else log_space
-        self.build_space = Context.DEFAULT_BUILD_SPACE + ss if ss or build_space is None else build_space
-        self.devel_space = Context.DEFAULT_DEVEL_SPACE + ss if ss or devel_space is None else devel_space
-        self.install_space = Context.DEFAULT_INSTALL_SPACE + ss if ss or install_space is None else install_space
         self.destdir = os.environ['DESTDIR'] if 'DESTDIR' in os.environ else None
 
         # Handle package whitelist/blacklist
         self.whitelist = whitelist or []
         self.blacklist = blacklist or []
 
+        # Handle default authors/maintainers
+        self.authors = authors or []
+        self.maintainers = maintainers or []
+        self.licenses = licenses or 'TODO'
+
         # Handle build options
         self.devel_layout = devel_layout if devel_layout else 'linked'
         self.install = install
@@ -308,6 +413,8 @@ class Context(object):
         self.env_cmake_prefix_path = None
         self.cmake_prefix_path = None
 
+        self.extends = extends
+
     def load_env(self):
 
         # Check for CMAKE_PREFIX_PATH in manual cmake args
@@ -411,20 +518,20 @@ class Context(object):
                 "--init`.")]
         if not self.source_space_exists():
             summary_warnings += [clr(
-                "Source space `@{yf}{_Context__source_space_abs}@|` does not yet exist.")]
+                "Source space `@{yf}{__source_space_abs}@|` does not yet exist.")]
 
+        spaces_summary = []
+        for space, space_dict in sorted(Context.SPACES.items()):
+            spaces_summary.append(
+                clr('@{cf}' + space_dict['space'] + ':@|' + ' ' * (18 - len(space_dict['space'])) +
+                    '{' + space + '_missing} @{yf}{__' + space + '_space_abs}@|'))
         summary = [
             [
                 clr("@{cf}Profile:@|                     @{yf}{profile}@|"),
                 clr("@{cf}Extending:@|        {extend_mode} @{yf}{extend}@|"),
                 clr("@{cf}Workspace:@|                   @{yf}{_Context__workspace}@|"),
             ],
-            [
-                clr("@{cf}Source Space:@|      {source_missing} @{yf}{_Context__source_space_abs}@|"),
-                clr("@{cf}Log Space:@|         {log_missing} @{yf}{_Context__log_space_abs}@|"),
-                clr("@{cf}Build Space:@|       {build_missing} @{yf}{_Context__build_space_abs}@|"),
-                clr("@{cf}Devel Space:@|       {devel_missing} @{yf}{_Context__devel_space_abs}@|"),
-                clr("@{cf}Install Space:@|     {install_missing} @{yf}{_Context__install_space_abs}@|"),
+            spaces_summary + [
                 clr("@{cf}DESTDIR:@|           {destdir_missing} @{yf}{_Context__destdir}@|")
             ],
             [
@@ -547,89 +654,10 @@ class Context(object):
                 raise ValueError("Resultspace path '{0}' does not exist.".format(value))
         self.__extend_path = value
 
-    @property
-    def source_space_abs(self):
-        return self.__source_space_abs
-
-    @property
-    def source_space(self):
-        return self.__source_space
-
-    @source_space.setter
-    def source_space(self, value):
-        if self.__locked:
-            raise RuntimeError("Setting of context members is not allowed while locked.")
-        self.__source_space = value
-        self.__source_space_abs = os.path.join(self.__workspace, value)
-
-    def source_space_exists(self):
-        "Returns true if the source space exists"
-        return os.path.exists(self.source_space_abs) and os.path.isdir(self.source_space_abs)
-
     def initialized(self):
         """Check if this context is initialized."""
         return self.workspace == find_enclosing_workspace(self.workspace)
 
-    @property
-    def log_space_abs(self):
-        return self.__log_space_abs
-
-    @property
-    def log_space(self):
-        return self.__log_space
-
-    @log_space.setter
-    def log_space(self, value):
-        if self.__locked:
-            raise RuntimeError("Setting of context members is not allowed while locked.")
-        self.__log_space = value
-        self.__log_space_abs = os.path.join(self.__workspace, value)
-
-    @property
-    def build_space_abs(self):
-        return self.__build_space_abs
-
-    @property
-    def build_space(self):
-        return self.__build_space
-
-    @build_space.setter
-    def build_space(self, value):
-        if self.__locked:
-            raise RuntimeError("Setting of context members is not allowed while locked.")
-        self.__build_space = value
-        self.__build_space_abs = os.path.join(self.__workspace, value)
-
-    @property
-    def devel_space_abs(self):
-        return self.__devel_space_abs
-
-    @property
-    def devel_space(self):
-        return self.__devel_space
-
-    @devel_space.setter
-    def devel_space(self, value):
-        if self.__locked:
-            raise RuntimeError("Setting of context members is not allowed while locked.")
-        self.__devel_space = value
-        self.__devel_space_abs = os.path.join(self.__workspace, value)
-
-    @property
-    def install_space_abs(self):
-        return self.__install_space_abs
-
-    @property
-    def install_space(self):
-        return self.__install_space
-
-    @install_space.setter
-    def install_space(self, value):
-        if self.__locked:
-            raise RuntimeError("Setting of context members is not allowed while locked.")
-        self.__install_space = value
-        self.__install_space_abs = os.path.join(self.__workspace, value)
-
     @property
     def destdir(self):
         return self.__destdir
@@ -772,6 +800,38 @@ class Context(object):
     def blacklist(self, value):
         self.__blacklist = value
 
+    @property
+    def authors(self):
+        return self.__authors
+
+    @authors.setter
+    def authors(self, value):
+        self.__authors = value
+
+    @property
+    def maintainers(self):
+        return self.__maintainers
+
+    @maintainers.setter
+    def maintainers(self, value):
+        self.__maintainers = value
+
+    @property
+    def licenses(self):
+        return self.__licenses
+
+    @licenses.setter
+    def licenses(self, value):
+        self.__licenses = value
+
+    @property
+    def extends(self):
+        return self.__extends
+
+    @extends.setter
+    def extends(self, value):
+        self.__extends = value
+
     @property
     def private_devel_path(self):
         """The path to the hidden directory in the develspace that
