@@ -20,6 +20,7 @@ except ImportError:
     from Queue import Empty
 
 import math
+import itertools
 import sys
 import threading
 import time
@@ -271,10 +272,10 @@ class ConsoleStatusController(threading.Thread):
 
         templates = {
             'successful': clr(" [@!@{gf}Successful@|] @{cf}{jid:<%d}@|" % max_jid_len),
-            'warned':     clr(" [    @!@{yf}Warned@|] @{cf}{jid:<%d}@|" % max_jid_len),
-            'failed':     clr(" [    @!@{rf}Failed@|] @{cf}{jid:<%d}@|" % max_jid_len),
-            'ignored':    clr(" [   @!@{kf}Ignored@|] @{cf}{jid:<%d}@|" % max_jid_len),
-            'abandoned':  clr(" [ @!@{rf}Abandoned@|] @{cf}{jid:<%d}@|" % max_jid_len),
+            'warned': clr(" [    @!@{yf}Warned@|] @{cf}{jid:<%d}@|" % max_jid_len),
+            'failed': clr(" [    @!@{rf}Failed@|] @{cf}{jid:<%d}@|" % max_jid_len),
+            'ignored': clr(" [   @!@{kf}Ignored@|] @{cf}{jid:<%d}@|" % max_jid_len),
+            'abandoned': clr(" [ @!@{rf}Abandoned@|] @{cf}{jid:<%d}@|" % max_jid_len),
         }
 
         # Calculate the maximum _printed_ length for each template
@@ -284,7 +285,7 @@ class ConsoleStatusController(threading.Thread):
         ])
 
         # Calculate the number of columns
-        number_of_columns = (terminal_width() / max_column_len) or 1
+        number_of_columns = int((terminal_width() / max_column_len) or 1)
 
         # Construct different categories of jobs (jid -> output template)
         successfuls = {}
@@ -319,11 +320,11 @@ class ConsoleStatusController(threading.Thread):
             wide_log(clr("[{}] Successful {}:").format(self.label, self.jobs_label))
             wide_log("")
             print_items_in_columns(
-                sorted(successfuls.items() + ignoreds.items()),
+                sorted(itertools.chain(successfuls.items(), ignoreds.items())),
                 number_of_columns)
         else:
             wide_log("")
-            wide_log(clr("[{}] No {} succeded.").format(self.label, self.jobs_label))
+            wide_log(clr("[{}] No {} succeeded.").format(self.label, self.jobs_label))
             wide_log("")
 
         # Print out whitelisted jobs
@@ -359,18 +360,21 @@ class ConsoleStatusController(threading.Thread):
     def print_compact_summary(self, completed_jobs, warned_jobs, failed_jobs):
         """Print a compact build summary."""
 
+        notification_title = ""
+        notification_msg = []
         # Print error summary
         if len(completed_jobs) == len(self.jobs) and all(completed_jobs.items()) and len(failed_jobs) == 0:
-            if self.show_notifications:
-                notify("{} Succeded".format(self.label.capitalize()),
-                       "{} {} completed with no warnings.".format(
-                    len(completed_jobs), self.jobs_label))
+            notification_title = "{} Succeeded".format(self.label.capitalize())
+            notification_msg.append("All {} {} succeeded!".format(len(self.jobs), self.jobs_label))
 
             wide_log(clr('[{}] Summary: All {} {} succeeded!').format(
                 self.label,
                 len(self.jobs),
                 self.jobs_label))
         else:
+            notification_msg.append("{} of {} {} succeeded.".format(
+                len([succeeded for jid, succeeded in completed_jobs.items() if succeeded]),
+                len(self.jobs), self.jobs_label))
             wide_log(clr('[{}] Summary: {} of {} {} succeeded.').format(
                 self.label,
                 len([succeeded for jid, succeeded in completed_jobs.items() if succeeded]),
@@ -383,6 +387,7 @@ class ConsoleStatusController(threading.Thread):
             wide_log(clr('[{}] Ignored: None.').format(
                 self.label))
         else:
+            notification_msg.append("{} {} were skipped.".format(len(all_ignored_jobs), self.jobs_label))
             wide_log(clr('[{}] Ignored: {} {} were skipped or are blacklisted.').format(
                 self.label,
                 len(all_ignored_jobs),
@@ -393,10 +398,8 @@ class ConsoleStatusController(threading.Thread):
             wide_log(clr('[{}] Warnings: None.').format(
                 self.label))
         else:
-            if self.show_notifications:
-                notify("{} Produced Warnings".format(self.label.capitalize()),
-                       "{} {} succeeded with warnings.".format(
-                    len(warned_jobs), self.jobs_label))
+            notification_title = "{} Succeeded with Warnings".format(self.label.capitalize())
+            notification_msg.append("{} {} succeeded with warnings.".format(len(warned_jobs), self.jobs_label))
 
             wide_log(clr('[{}] Warnings: {} {} succeeded with warnings.').format(
                 self.label,
@@ -410,10 +413,8 @@ class ConsoleStatusController(threading.Thread):
                 self.label,
                 self.jobs_label))
         else:
-            if self.show_notifications:
-                notify("{} Incomplete".format(self.label.capitalize()),
-                       "{} {} were abandoned.".format(
-                    len(all_abandoned_jobs), self.jobs_label))
+            notification_title = "{} Incomplete".format(self.label.capitalize())
+            notification_msg.append("{} {} were abandoned.".format(len(all_abandoned_jobs), self.jobs_label))
 
             wide_log(clr('[{}] Abandoned: {} {} were abandoned.').format(
                 self.label,
@@ -426,16 +427,22 @@ class ConsoleStatusController(threading.Thread):
                 self.label,
                 self.jobs_label))
         else:
-            if self.show_notifications:
-                notify("{} Failed".format(self.label.capitalize()),
-                       "{} {} failed.".format(
-                    len(failed_jobs), self.jobs_label))
+            notification_title = "{} Failed".format(self.label.capitalize())
+            notification_msg.append("{} {} failed.".format(len(failed_jobs), self.jobs_label))
 
             wide_log(clr('[{}] Failed: {} {} failed.').format(
                 self.label,
                 len(failed_jobs),
                 self.jobs_label))
 
+        if self.show_notifications:
+            if len(failed_jobs) != 0:
+                notify(notification_title, "\n".join(notification_msg), icon_image='catkin_icon_red.png')
+            elif len(warned_jobs) != 0:
+                notify(notification_title, "\n".join(notification_msg), icon_image='catkin_icon_yellow.png')
+            else:
+                notify(notification_title, "\n".join(notification_msg))
+
     def run(self):
         queued_jobs = []
         active_jobs = []
@@ -448,6 +455,13 @@ class ConsoleStatusController(threading.Thread):
         active_stages = dict()
 
         start_time = self.pre_start_time or time.time()
+        last_update_time = time.time()
+
+        # If the status rate is too low, just disable it
+        if self.active_status_rate < 1E-3:
+            self.show_active_status = False
+        else:
+            update_duration = 1.0 / self.active_status_rate
 
         # Disable the wide log padding if the status is disabled
         if not self.show_active_status:
@@ -458,55 +472,68 @@ class ConsoleStatusController(threading.Thread):
             if not self.keep_running:
                 wide_log(clr('[{}] An internal error occurred!').format(self.label))
                 return
+
             # Write a continuously-updated status line
             if self.show_active_status:
+
                 # Try to get an event from the queue (non-blocking)
                 try:
                     event = self.event_queue.get(False)
                 except Empty:
-                    # Print live status (overwrites last line)
-                    status_line = clr('[{} {} s] [{}/{} complete] [{}/{} jobs] [{} queued]').format(
-                        self.label,
-                        format_time_delta_short(time.time() - start_time),
-                        len(completed_jobs),
-                        len(self.jobs),
-                        job_server.running_jobs(),
-                        job_server.max_jobs(),
-                        len(queued_jobs) + len(active_jobs) - len(active_stages)
-                    )
+                    # Determine if the status should be shown based on the desired
+                    # status rate
+                    elapsed_time = time.time() - last_update_time
+                    show_status_now = elapsed_time > update_duration
 
-                    # Show failed jobs
-                    if len(failed_jobs) > 0:
-                        status_line += clr(' [@!@{rf}{}@| @{rf}failed@|]').format(len(failed_jobs))
+                    if show_status_now:
+                        # Print live status (overwrites last line)
+                        status_line = clr('[{} {} s] [{}/{} complete] [{}/{} jobs] [{} queued]').format(
+                            self.label,
+                            format_time_delta_short(time.time() - start_time),
+                            len(completed_jobs),
+                            len(self.jobs),
+                            job_server.running_jobs(),
+                            job_server.max_jobs(),
+                            len(queued_jobs) + len(active_jobs) - len(active_stages)
+                        )
 
-                    # Check load / mem
-                    if not job_server.load_ok():
-                        status_line += clr(' [@!@{rf}High Load@|]')
-                    if not job_server.mem_ok():
-                        status_line += clr(' [@!@{rf}Low Memory@|]')
+                        # Show failed jobs
+                        if len(failed_jobs) > 0:
+                            status_line += clr(' [@!@{rf}{}@| @{rf}failed@|]').format(len(failed_jobs))
 
-                    # Add active jobs
-                    if len(active_jobs) == 0:
-                        status_line += clr(' @/@!@{kf}Waiting for jobs...@|')
+                        # Check load / mem
+                        if not job_server.load_ok():
+                            status_line += clr(' [@!@{rf}High Load@|]')
+                        if not job_server.mem_ok():
+                            status_line += clr(' [@!@{rf}Low Memory@|]')
+
+                        # Add active jobs
+                        if len(active_jobs) == 0:
+                            status_line += clr(' @/@!@{kf}Waiting for jobs...@|')
+                        else:
+                            active_labels = []
+
+                            for j, (s, t, p) in active_stages.items():
+                                d = format_time_delta_short(cumulative_times[j] + time.time() - t)
+                                if p == '':
+                                    active_labels.append(clr('[{}:{} - {}]').format(j, s, d))
+                                else:
+                                    active_labels.append(clr('[{}:{} ({}%) - {}]').format(j, s, p, d))
+
+                            status_line += ' ' + ' '.join(active_labels)
+
+                        # Print the status line
+                        # wide_log(status_line)
+                        wide_log(status_line, rhs='', end='\r')
+                        sys.stdout.flush()
+
+                        # Store this update time
+                        last_update_time = time.time()
                     else:
-                        active_labels = []
+                        time.sleep(max(0.0, min(update_duration - elapsed_time, 0.01)))
 
-                        for j, (s, t, p) in active_stages.items():
-                            d = format_time_delta_short(cumulative_times[j] + time.time() - t)
-                            if p == '':
-                                active_labels.append(clr('[{}:{} - {}]').format(j, s, d))
-                            else:
-                                active_labels.append(clr('[{}:{} ({}%) - {}]').format(j, s, p, d))
-
-                        status_line += ' ' + ' '.join(active_labels)
-
-                    # Print the status line
-                    # wide_log(status_line)
-                    wide_log(status_line, rhs='', end='\r')
-                    sys.stdout.flush()
-                    if self.active_status_rate > 1E-5:
-                        time.sleep(1.0 / self.active_status_rate)
-                        continue
+                    # Only continue when no event was received
+                    continue
             else:
                 # Try to get an event from the queue (blocking)
                 try:
@@ -600,10 +627,14 @@ class ConsoleStatusController(threading.Thread):
                 del active_stages[event.data['job_id']]
 
                 header_border = None
+                header_border_file = sys.stdout
                 header_title = None
+                header_title_file = sys.stdout
                 lines = []
                 footer_title = None
+                footer_title_file = sys.stdout
                 footer_border = None
+                footer_border_file = sys.stdout
 
                 # Generate headers / borders for output
                 if event.data['succeeded']:
@@ -619,12 +650,15 @@ class ConsoleStatusController(threading.Thread):
 
                         # Output contains warnings
                         header_border = clr('@!@{yf}' + '_' * (terminal_width() - 1) + '@|')
+                        header_border_file = sys.stderr
                         header_title = clr(
                             'Warnings << {}:{} {}').format(
                                 event.data['job_id'],
                                 event.data['stage_label'],
                                 event.data['logfile_filename'])
+                        header_title_file = sys.stderr
                         footer_border = clr('@{yf}' + '.' * (terminal_width() - 1) + '@|')
+                        footer_border_file = sys.stderr
                     else:
                         # Normal output, no warnings
                         header_title = clr(
@@ -639,12 +673,15 @@ class ConsoleStatusController(threading.Thread):
                 else:
                     # Output contains errors
                     header_border = clr('@!@{rf}' + '_' * (terminal_width() - 1) + '@|')
+                    header_border_file = sys.stderr
                     header_title = clr(
                         'Errors << {}:{} {}').format(
                             event.data['job_id'],
                             event.data['stage_label'],
                             event.data['logfile_filename'])
+                    header_title_file = sys.stderr
                     footer_border = clr('@{rf}' + '.' * (terminal_width() - 1) + '@|')
+                    footer_border_file = sys.stderr
 
                     footer_title = clr(
                         'Failed << {}:{:<{}} [ Exited with code {} ]').format(
@@ -652,13 +689,14 @@ class ConsoleStatusController(threading.Thread):
                             event.data['stage_label'],
                             max(0, self.max_jid_length - len(event.data['job_id'])),
                             event.data['retcode'])
+                    footer_title_file = sys.stderr
 
+                lines_target = sys.stdout
                 if self.show_buffered_stdout:
                     if len(event.data['interleaved']) > 0:
                         lines = [
-                            l
-                            for l in event.data['interleaved'].splitlines(True)
-                            if (self.show_compact_io is False or len(l.strip()) > 0)
+                            line for line in event.data['interleaved'].splitlines(True)
+                            if (self.show_compact_io is False or len(line.strip()) > 0)
                         ]
                     else:
                         header_border = None
@@ -667,10 +705,10 @@ class ConsoleStatusController(threading.Thread):
                 elif self.show_buffered_stderr:
                     if len(event.data['stderr']) > 0:
                         lines = [
-                            l
-                            for l in event.data['stderr'].splitlines(True)
-                            if (self.show_compact_io is False or len(l.strip()) > 0)
+                            line for line in event.data['stderr'].splitlines(True)
+                            if (self.show_compact_io is False or len(line.strip()) > 0)
                         ]
+                        lines_target = sys.stderr
                     else:
                         header_border = None
                         header_title = None
@@ -683,19 +721,19 @@ class ConsoleStatusController(threading.Thread):
 
                     # Print the output
                     if header_border:
-                        wide_log(header_border)
+                        wide_log(header_border, file=header_border_file)
                     if header_title:
-                        wide_log(header_title)
+                        wide_log(header_title, file=header_title_file)
                     if len(lines) > 0:
-                        wide_log(''.join(lines), end='\r')
+                        wide_log(''.join(lines), end='\r', file=lines_target)
                     if footer_border:
-                        wide_log(footer_border)
+                        wide_log(footer_border, file=footer_border_file)
                     if footer_title:
-                        wide_log(footer_title)
+                        wide_log(footer_title, file=footer_title_file)
 
             elif 'STDERR' == eid:
                 if self.show_live_stderr and len(event.data['data']) > 0:
-                    wide_log(self.format_interleaved_lines(event.data), end='\r')
+                    wide_log(self.format_interleaved_lines(event.data), end='\r', file=sys.stderr)
 
             elif 'STDOUT' == eid:
                 if self.show_live_stdout and len(event.data['data']) > 0:
@@ -728,4 +766,4 @@ class ConsoleStatusController(threading.Thread):
         template = '\r{}\r{}'.format(' ' * terminal_width(), prefix)
         suffix = clr('@|')
 
-        return ''.join(template + l + suffix for l in data['data'].splitlines(True))
+        return ''.join(template + line + suffix for line in data['data'].splitlines(True))
