@@ -16,13 +16,12 @@
 
 import os
 import pkg_resources
-import stat
 import sys
 import time
 import traceback
 import yaml
 
-import trollius as asyncio
+import asyncio
 
 try:
     # Python3
@@ -42,9 +41,8 @@ except ImportError as e:
 
 from catkin_pkg.package import parse_package
 
-from catkin_tools.common import FakeLock
+from catkin_tools.common import FakeLock, expand_glob_package
 from catkin_tools.common import format_time_delta
-from catkin_tools.common import get_build_type
 from catkin_tools.common import get_cached_recursive_build_depends_in_workspace
 from catkin_tools.common import get_recursive_run_depends_in_workspace
 from catkin_tools.common import log
@@ -92,13 +90,20 @@ def determine_packages_to_be_built(packages, context, workspace_packages):
     packages_to_be_built = []
     packages_to_be_built_deps = []
 
+    workspace_package_names = dict([(pkg.name, (path, pkg)) for path, pkg in ordered_packages])
     # Determine the packages to be built
     if packages:
         # First assert all of the packages given are in the workspace
-        workspace_package_names = dict([(pkg.name, (path, pkg)) for path, pkg in ordered_packages])
         for package in packages:
             if package not in workspace_package_names:
-                sys.exit("[build] Given package '{0}' is not in the workspace".format(package))
+                # Try whether package is a pattern and matches
+                glob_packages = expand_glob_package(package, workspace_package_names)
+                if len(glob_packages) > 0:
+                    packages.extend(glob_packages)
+                    continue
+                else:
+                    sys.exit("[build] Given package '{0}' is not in the workspace"
+                             "and pattern does not match any package".format(package))
             # If metapackage, include run depends which are in the workspace
             package_obj = workspace_package_names[package][1]
             if 'metapackage' in [e.tagname for e in package_obj.exports]:
@@ -115,19 +120,27 @@ def determine_packages_to_be_built(packages, context, workspace_packages):
     else:
         # Only use whitelist when no other packages are specified
         if len(context.whitelist) > 0:
-            packages_to_be_built = [p for p in ordered_packages if (p[1].name in context.whitelist)]
+            # Expand glob patterns in whitelist
+            whitelist = []
+            for whitelisted_package in context.whitelist:
+                whitelist.extend(expand_glob_package(whitelisted_package, workspace_package_names))
+            packages_to_be_built = [p for p in ordered_packages if (p[1].name in whitelist)]
         else:
             packages_to_be_built = ordered_packages
 
     # Filter packages with blacklist
     if len(context.blacklist) > 0:
+        # Expand glob patterns in blacklist
+        blacklist = []
+        for blacklisted_package in context.blacklist:
+            blacklist.extend(expand_glob_package(blacklisted_package, workspace_package_names))
+        # Apply blacklist to packages and dependencies
         packages_to_be_built = [
             (path, pkg) for path, pkg in packages_to_be_built
-            if (pkg.name not in context.blacklist or pkg.name in packages)]
+            if (pkg.name not in blacklist or pkg.name in packages)]
         packages_to_be_built_deps = [
             (path, pkg) for path, pkg in packages_to_be_built_deps
-            if (pkg.name not in context.blacklist or pkg.name in packages)]
-        ordered_packages = ordered_packages
+            if (pkg.name not in blacklist or pkg.name in packages)]
 
     return packages_to_be_built, packages_to_be_built_deps, ordered_packages
 
@@ -153,9 +166,8 @@ def get_built_unbuilt_packages(context, workspace_packages):
     # Get names of all unbuilt packages
     unbuilt_pkgs = set()
     for path, pkg in workspace_packages.items():
-        if 'metapackage' not in [e.tagname for e in pkg.exports]:
-            if pkg.name not in built_packages:
-                unbuilt_pkgs.add(pkg.name)
+        if pkg.name not in built_packages:
+            unbuilt_pkgs.add(pkg.name)
 
     return built_packages, unbuilt_pkgs
 
@@ -240,7 +252,7 @@ def build_isolated_workspace(
     # Check build config
     if os.path.exists(os.path.join(context.build_space_abs, BUILDSPACE_MARKER_FILE)):
         with open(os.path.join(context.build_space_abs, BUILDSPACE_MARKER_FILE)) as buildspace_marker_file:
-            existing_buildspace_marker_data = yaml.load(buildspace_marker_file)
+            existing_buildspace_marker_data = yaml.safe_load(buildspace_marker_file)
             misconfig_lines = ''
             for (k, v) in existing_buildspace_marker_data.items():
                 new_v = buildspace_marker_data.get(k, None)
@@ -267,9 +279,10 @@ def build_isolated_workspace(
     if os.path.exists(context.build_space_abs):
         if os.path.isfile(context.build_space_abs):
             sys.exit(clr(
-                "[build] @{rf}Error:@| Build space '{0}' exists but is a file and not a folder."
+                "[build] @{rf}Error:@| " +
+                "Build space '{0}' exists but is a file and not a folder."
                 .format(context.build_space_abs)))
-    # If it dosen't exist, create it
+    # If it doesn't exist, create it
     else:
         log("[build] Creating build space: '{0}'".format(context.build_space_abs))
         os.makedirs(context.build_space_abs)
@@ -459,18 +472,14 @@ def build_isolated_workspace(
 
     # It's a problem if there aren't any build types available
     if len(build_job_creators) == 0:
-        sys.exit('Error: No build types availalbe. Please check your catkin_tools installation.')
+        sys.exit('Error: No build types available. Please check your catkin_tools installation.')
 
     # Construct jobs
     for pkg_path, pkg in all_packages:
         if pkg.name not in packages_to_be_built_names:
             continue
 
-        # Ignore metapackages
-        if 'metapackage' in [e.tagname for e in pkg.exports]:
-            continue
-
-        # Get actual execution deps
+        # Get actual build deps
         deps = [
             p.name for _, p
             in get_cached_recursive_build_depends_in_workspace(pkg, packages_to_be_built)
@@ -491,7 +500,7 @@ def build_isolated_workspace(
             pre_clean=pre_clean)
 
         # Create the job based on the build type
-        build_type = get_build_type(pkg)
+        build_type = pkg.get_build_type()
 
         if build_type in build_job_creators:
             jobs.append(build_job_creators[build_type](**build_job_kwargs))
@@ -577,6 +586,8 @@ def build_isolated_workspace(
         wide_log("[build] Interrupted by user!")
         event_queue.put(None)
 
+        return 130  # EOWNERDEAD return code is not part of the errno module.
+
 
 def _create_unmerged_devel_setup(context, unbuilt):
     # Find all of the leaf packages in the workspace
@@ -630,19 +641,16 @@ def _create_unmerged_devel_setup(context, unbuilt):
     )
     with open(setup_sh_path, 'w') as f:
         f.write(env_file)
-    os.chmod(setup_sh_path, stat.S_IXUSR | stat.S_IWUSR | stat.S_IRUSR)
 
     # Create setup.bash file
     setup_bash_path = os.path.join(context.devel_space_abs, 'setup.bash')
     with open(setup_bash_path, 'w') as f:
         f.write(SETUP_BASH_TEMPLATE)
-    os.chmod(setup_bash_path, stat.S_IXUSR | stat.S_IWUSR | stat.S_IRUSR)
 
     # Create setup.zsh file
     setup_zsh_path = os.path.join(context.devel_space_abs, 'setup.zsh')
     with open(setup_zsh_path, 'w') as f:
         f.write(SETUP_ZSH_TEMPLATE)
-    os.chmod(setup_zsh_path, stat.S_IXUSR | stat.S_IWUSR | stat.S_IRUSR)
 
 
 def _create_unmerged_devel_setup_for_install(context):
