@@ -37,7 +37,6 @@ from catkin_tools.argument_parsing import add_context_args
 from catkin_tools.argument_parsing import add_cmake_and_make_and_catkin_make_args
 from catkin_tools.argument_parsing import configure_make_args
 
-from catkin_tools.common import get_build_type
 from catkin_tools.common import getcwd
 from catkin_tools.common import is_tty
 from catkin_tools.common import log
@@ -48,7 +47,8 @@ from catkin_tools.context import Context
 
 import catkin_tools.execution.job_server as job_server
 
-from catkin_tools.jobs.utils import get_env_loader
+from catkin_tools.jobs.utils import CommandMissing
+from catkin_tools.jobs.utils import loadenv
 
 from catkin_tools.metadata import find_enclosing_workspace
 from catkin_tools.metadata import get_metadata
@@ -82,6 +82,7 @@ def fixed__ArgumentGroup___init__(self, container, title=None, description=None,
     # Make sure this line is run, maybe redundant on versions which already have it
     self._mutually_exclusive_groups = container._mutually_exclusive_groups
 
+
 # Monkey patch in the fixed constructor
 argparse._ArgumentGroup.__init__ = fixed__ArgumentGroup___init__
 
@@ -213,7 +214,7 @@ def dry_run(context, packages, no_deps, start_with):
     max_name_len = str(max([len(pkg.name) for pth, pkg in packages_to_be_built]))
     prefix = clr('@{pf}' + ('------ ' if start_with else '- ') + '@|')
     for pkg_path, pkg in packages_to_be_built:
-        build_type = get_build_type(pkg)
+        build_type = pkg.get_build_type()
         if build_type == 'catkin' and 'metapackage' in [e.tagname for e in pkg.exports]:
             build_type = 'metapackage'
         if start_with and pkg.name == start_with:
@@ -228,7 +229,8 @@ def print_build_env(context, package_name):
     # Load the environment used by this package for building
     for pth, pkg in workspace_packages.items():
         if pkg.name == package_name:
-            environ = get_env_loader(pkg, context)(os.environ)
+            environ = dict(os.environ)
+            loadenv(None, None, environ, pkg, context)
             print(format_env_dict(environ))
             return 0
     print('[build] Error: Package `{}` not in workspace.'.format(package_name),
@@ -395,25 +397,28 @@ def main(opts):
         parallel_jobs = None
 
     # Set VERBOSE environment variable
-    if opts.verbose:
+    if opts.verbose and 'VERBOSE' not in os.environ:
         os.environ['VERBOSE'] = '1'
 
-    return build_isolated_workspace(
-        ctx,
-        packages=opts.packages,
-        start_with=opts.start_with,
-        no_deps=opts.no_deps,
-        unbuilt=opts.unbuilt,
-        n_jobs=parallel_jobs,
-        force_cmake=opts.force_cmake,
-        pre_clean=opts.pre_clean,
-        force_color=opts.force_color,
-        quiet=not opts.verbose,
-        interleave_output=opts.interleave_output,
-        no_status=opts.no_status,
-        limit_status_rate=opts.limit_status_rate,
-        lock_install=not opts.no_install_lock,
-        no_notify=opts.no_notify,
-        continue_on_failure=opts.continue_on_failure,
-        summarize_build=opts.summarize  # Can be True, False, or None
-    )
+    try:
+        return build_isolated_workspace(
+            ctx,
+            packages=opts.packages,
+            start_with=opts.start_with,
+            no_deps=opts.no_deps,
+            unbuilt=opts.unbuilt,
+            n_jobs=parallel_jobs,
+            force_cmake=opts.force_cmake,
+            pre_clean=opts.pre_clean,
+            force_color=opts.force_color,
+            quiet=not opts.verbose,
+            interleave_output=opts.interleave_output,
+            no_status=opts.no_status,
+            limit_status_rate=opts.limit_status_rate,
+            lock_install=not opts.no_install_lock,
+            no_notify=opts.no_notify,
+            continue_on_failure=opts.continue_on_failure,
+            summarize_build=opts.summarize  # Can be True, False, or None
+        )
+    except CommandMissing as e:
+        sys.exit(clr("[build] @!@{rf}Error:@| {0}").format(e))
